package com.dgtly.system.service.impl;

import java.util.List;
import java.util.ArrayList;
import com.dgtly.common.core.domain.Ztree;
import com.dgtly.common.utils.DateUtils;
import com.dgtly.common.utils.StringUtils;
import com.dgtly.system.domain.*;
import com.dgtly.system.mapper.SysUserAuthorRelMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.dgtly.system.mapper.SysUserOrderAuthorMapper;
import com.dgtly.system.service.ISysUserOrderAuthorService;
import com.dgtly.common.core.text.Convert;

/**
 * 权限Service业务层处理
 * 
 * @author dgtly
 * @date 2022-03-22
 */
@Service
public class SysUserOrderAuthorServiceImpl implements ISysUserOrderAuthorService 
{
    @Autowired
    private SysUserOrderAuthorMapper sysUserOrderAuthorMapper;

    @Autowired
    private SysUserAuthorRelMapper userAuthorRelMapper;

    /**
     * 查询权限
     * 
     * @param authorId 权限ID
     * @return 权限
     */
    @Override
    public SysUserOrderAuthor selectSysUserOrderAuthorById(Long authorId)
    {
        return sysUserOrderAuthorMapper.selectSysUserOrderAuthorById(authorId);
    }

    /**
     * 查询权限列表
     * 
     * @param sysUserOrderAuthor 权限
     * @return 权限
     */
    @Override
    public List<SysUserOrderAuthor> selectSysUserOrderAuthorList(SysUserOrderAuthor sysUserOrderAuthor)
    {
        return sysUserOrderAuthorMapper.selectSysUserOrderAuthorList(sysUserOrderAuthor);
    }

    /**
     * 新增权限
     * 
     * @param sysUserOrderAuthor 权限
     * @return 结果
     */
    @Override
    public int insertSysUserOrderAuthor(SysUserOrderAuthor sysUserOrderAuthor)
    {
        sysUserOrderAuthor.setCreateTime(DateUtils.getNowDate());
        return sysUserOrderAuthorMapper.insertSysUserOrderAuthor(sysUserOrderAuthor);
    }

    /**
     * 修改权限
     * 
     * @param sysUserOrderAuthor 权限
     * @return 结果
     */
    @Override
    public int updateSysUserOrderAuthor(SysUserOrderAuthor sysUserOrderAuthor)
    {
        sysUserOrderAuthor.setUpdateTime(DateUtils.getNowDate());
        return sysUserOrderAuthorMapper.updateSysUserOrderAuthor(sysUserOrderAuthor);
    }

    /**
     * 删除权限对象
     * 
     * @param ids 需要删除的数据ID
     * @return 结果
     */
    @Override
    public int deleteSysUserOrderAuthorByIds(String ids)
    {
        return sysUserOrderAuthorMapper.deleteSysUserOrderAuthorByIds(Convert.toStrArray(ids));
    }

    /**
     * 删除权限信息
     * 
     * @param authorId 权限ID
     * @return 结果
     */
    @Override
    public int deleteSysUserOrderAuthorById(Long authorId)
    {
        return sysUserOrderAuthorMapper.deleteSysUserOrderAuthorById(authorId);
    }

    public int insertRel(SysUserAuthorRel role)
    {
        userAuthorRelMapper.deleteSysUserAuthorRelById(role.getUserId());
        int rows = 1;
        // 新增角色与部门（数据权限）管理
        List<SysUserAuthorRel> list = new ArrayList<SysUserAuthorRel>();
        for (Long authorId : role.getAuthorIds())
        {
            SysUserAuthorRel rd = new SysUserAuthorRel();
            rd.setUserId(role.getUserId());
            rd.setAuthorId(authorId);
            list.add(rd);
        }
        if (list.size() > 0)
        {
            rows = userAuthorRelMapper.batchRel(list);
        }
        return rows;
    }

    /**
     * 查询权限树列表
     * 
     * @return 所有权限信息
     */
    @Override
    public List<Ztree> selectSysUserOrderAuthorTree()
    {
        List<SysUserOrderAuthor> sysUserOrderAuthorList = sysUserOrderAuthorMapper.selectSysUserOrderAuthorList(new SysUserOrderAuthor());
        List<Ztree> ztrees = new ArrayList<Ztree>();
        for (SysUserOrderAuthor sysUserOrderAuthor : sysUserOrderAuthorList)
        {
            Ztree ztree = new Ztree();
            ztree.setId(sysUserOrderAuthor.getAuthorId());
            ztree.setpId(sysUserOrderAuthor.getParentId());
            ztree.setName(sysUserOrderAuthor.getAuthorName());
            ztree.setTitle(sysUserOrderAuthor.getAuthorName());
            ztrees.add(ztree);
        }
        return ztrees;
    }

    public List<Ztree> userAuthorTreeData(Long userId) {
        List<Ztree> ztrees = new ArrayList<Ztree>();
        List<SysUserOrderAuthor> menuList = sysUserOrderAuthorMapper.selectSysUserOrderAuthorList(new SysUserOrderAuthor());
        List<Long> user = sysUserOrderAuthorMapper.selectAuthorTree(userId);
        ztrees = initTree(menuList, user);

        getAuthorByLevel();//todo
        return ztrees;
    }

    /**
     * 对象转树
     * @param sysUserOrderAuthorList
     * @return
     */
    public List<Ztree> initTree(List<SysUserOrderAuthor> sysUserOrderAuthorList, List<Long> roleMenuList)
    {
        List<Ztree> ztrees = new ArrayList<Ztree>();
        boolean isCheck = StringUtils.isNotNull(roleMenuList);
        for (SysUserOrderAuthor menu : sysUserOrderAuthorList)
        {
            Ztree ztree = new Ztree();
            ztree.setId(menu.getAuthorId());
            ztree.setpId(menu.getParentId());
            ztree.setName(menu.getAuthorName());
            ztree.setTitle(menu.getAuthorName());
            if (isCheck)
            {
                ztree.setChecked(roleMenuList.contains(menu.getAuthorId()));
            }
            ztree.setOpen(true);
            ztrees.add(ztree);
        }
        return ztrees;
    }

    public void userAuthorTask(){

    }

    public void getAuthorByLevel(){
        List<SysUserOrderAuthor> list = sysUserOrderAuthorMapper.selectSysUserOrderAuthorList(new SysUserOrderAuthor());
        List<Long> ids = new ArrayList<>();
        getParentAuthor(ids, list, 8l);
        System.out.println(ids);
    }

    public void getParentAuthor(List<Long> ids, List<SysUserOrderAuthor> list, Long id){
        for (SysUserOrderAuthor sysUserOrderAuthor : list) {
            Long itemId = sysUserOrderAuthor.getAuthorId();
            if(itemId.equals(0l)){
                ids.add(itemId);
            }else if(itemId.equals(id)){
                ids.add(itemId);
                getParentAuthor(ids, list , sysUserOrderAuthor.getParentId());
            }
        }
    }
}
